﻿using DemoApiDwservice.Models;
using DemoApiDwservice.Models.Request;
using DemoApiDwservice.Models.Response;
using DemoApiDwservice.Services;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace DemoApiDwservice.Controllers
{
    public class AccountController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }
        public async Task<IActionResult> GetAccountInfo()
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                Account account = await dwServiceDataStore.getAccountInfo();
                if(account.status == 200 || account.status==0)
                    return View(account);
                else
                    return View("~/Views/Shared/Error.cshtml");
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml"); 
            }
        }

        public IActionResult AddServices()
        {
            return View();
        }

        [HttpPost]
        public ActionResult AddServices(ServiceRequest req)
        {
            if(req.agents == 0 && req.channelsBasic==0)
                return View("~/Views/Shared/Error.cshtml");
            else
                return RedirectToAction("AddServicesResponse", req);            
        }

        public async Task<ActionResult> AddServicesResponse(ServiceRequest req)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                ServiceResponse serviceResponse = await dwServiceDataStore.addService(req);
                return View(serviceResponse);
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }


        public IActionResult RemoveServices()
        {
            return View();
        }

        [HttpPost]
        public ActionResult RemoveServices(RemoveServiceViewModel req)
        {
            try
            {
                if(req.agents == 0 && string.IsNullOrEmpty(req.channelsIDs))
                    return View("~/Views/Shared/Error.cshtml");
                RemoveServiceRequest removeServiceRequest = new RemoveServiceRequest();
                List<int> idList = new List<int>();
                if (req.channelsIDs != null)
                {
                    string[] idstring = req.channelsIDs.Split(',');
                    foreach (string s in idstring)
                    {
                        if (!string.IsNullOrEmpty(s.Trim()))
                        {
                            int id = Convert.ToInt32(s);
                            idList.Add(id);
                        }
                    }
                }

                removeServiceRequest.agents = req.agents;
                removeServiceRequest.channelsIDs = idList.ToArray();
                return RedirectToAction("RemoveServicesResponse", removeServiceRequest);
            }
            catch (Exception)
            {

                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public async Task<ActionResult> RemoveServicesResponse(RemoveServiceRequest req)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                RemoveServiceResponse removeServiceResponse = await dwServiceDataStore.removeService(req);

                return View(removeServiceResponse);
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public IActionResult GetCreditTransactions()
        {
            return View();

        }

        [HttpPost]
        public IActionResult GetCreditTransactions(int days)
        {
            try
            {
                TransactionRequest transactionRequest = new TransactionRequest();
                transactionRequest.days = days;
                return RedirectToAction("GetCreditTransactionsResponse", transactionRequest);

            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public async Task<ActionResult> GetCreditTransactionsResponse(TransactionRequest req)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                List<TransactionsResponse> transactionsResponse = await dwServiceDataStore.getCreditTransactions(req.days);
                return View(transactionsResponse);

            }
            catch (Exception)
            {

                return View("~/Views/Shared/Error.cshtml");
            }
        }
    }
}