﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using DemoApiDwservice.Models;
using DemoApiDwservice.Models.Request;
using DemoApiDwservice.Models.Response;
using DemoApiDwservice.Services;
using Microsoft.AspNetCore.Mvc;

namespace DemoApiDwservice.Controllers
{
    public class AgentsController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        public IActionResult CreateAnAgent()
        {
            return View();
        }

        [HttpPost]
        public IActionResult CreateAnAgent(AgentRequest req)
        {
            if(string.IsNullOrEmpty(req.name) && string.IsNullOrEmpty(req.description))
                return View("~/Views/Shared/Error.cshtml");
            else
                return RedirectToAction("CreateAnAgentResponse", req);
        }

        public async Task<ActionResult> CreateAnAgentResponse(AgentRequest req)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                Agent agent = await dwServiceDataStore.newAgent(req);

                return View(agent);
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public IActionResult ModifyAnAgent()
        {
            return View();
        }

        [HttpPost]
        public async Task<ActionResult> ModifyAnAgent(ModifyAgentViewModel req)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                AgentRequest agentRequest = new AgentRequest();
                agentRequest.name = req.name;
                agentRequest.description = req.description;
                BaseResponse baseResponse = await dwServiceDataStore.modifyAgent(req.id, agentRequest);
                if (baseResponse.status == 200)
                    return View("Index");
                else
                    return View("~/Views/Shared/Error.cshtml");
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public IActionResult DeleteAnAgent()
        {
            return View();
        }

        [HttpPost]
        public async Task<ActionResult> DeleteAnAgent(string id)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();

                BaseResponse baseResponse = await dwServiceDataStore.deleteAgent(id);
                if (baseResponse.status == 200)
                    return View("Index");
                else
                    return View("~/Views/Shared/Error.cshtml");
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public async Task<ActionResult> ListAllAgents()
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();

                List<Agent> agents = await dwServiceDataStore.getAgentsList();
                return View(agents);
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public IActionResult GetAgentByIdName()
        {
            return View();
        }

        [HttpPost]
        public IActionResult GetAgentById(string id)
        {
            try
            {
                if(string.IsNullOrEmpty(id))
                    return View("~/Views/Shared/Error.cshtml");
                GetAgentRequest getAgentRequest = new GetAgentRequest();
                getAgentRequest.id = id;
                return RedirectToAction("GetAgent", getAgentRequest);

            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        [HttpPost]
        public IActionResult GetAgentByName(string name)
        {
            try
            {
                if (string.IsNullOrEmpty(name))
                    return View("~/Views/Shared/Error.cshtml");
                GetAgentRequest getAgentRequest = new GetAgentRequest();
                getAgentRequest.name = name;
                return RedirectToAction("GetAgent", getAgentRequest);

            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }


        public async Task<ActionResult> GetAgent(GetAgentRequest req)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();

                Agent agent;
                if (req.id != null)
                    agent = await dwServiceDataStore.getAgentbyId(req.id);
                else
                    agent = await dwServiceDataStore.getAgentbyName(req.name);
                return View(agent);

            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }
    }


}