﻿using DemoApiDwservice.Models;
using DemoApiDwservice.Models.Request;
using DemoApiDwservice.Models.Response;
using DemoApiDwservice.Services;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using Microsoft.CodeAnalysis.Editing;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace DemoApiDwservice.Controllers
{
    public class SessionsController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        public IActionResult CreateANewSession()
        {
            return View();
        }

        [HttpPost]
        public async Task<ActionResult> CreateANewSession(CreateASessionViewModel model)
        {
            try
            {
                if(string.IsNullOrEmpty(model.idAgent) || (model.fullAccess == false &&(model.getDesktop  == false
                    && model.getFileSystem == false && model.getLogWatch == false && model.getResoruce == false && model.getTextEditor == false)))
                    return View("~/Views/Shared/Error.cshtml");

                int channel = await Helpers.Helpers.getChannelsFree();
                if (channel != -1)
                {
                    DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                    DesktopResponse response = new DesktopResponse();
                    DesktopRequest desktopRequest = new DesktopRequest();
                    desktopRequest.idChannel = channel;
                    desktopRequest.idAgent = model.idAgent;
                    desktopRequest.locale = model.locale;
                    desktopRequest.fullAccess = model.fullAccess;
                    desktopRequest.hideAppsBar = model.hideAppsBar;
                    desktopRequest.postMessageOrigin = "https://" + Request.Host.Value;
                    desktopRequest.showAppOnLoad = model.showAppOnLoad;
                    desktopRequest.applications = new Applications();
                    if (model.getDesktop)
                    {
                        Desktop desktop = new Desktop();
                        desktop.fullAccess = true;
                        desktopRequest.applications.desktop = desktop;
                    }
                    if (model.getFileSystem)
                    {
                        Filesystem filesystem = new Filesystem();
                        filesystem.fullAccess = true;
                        desktopRequest.applications.filesystem = filesystem;
                    }
                    if (model.getResoruce)
                    {
                        Resource resource = new Resource();
                        resource.fullAccess = true;
                        desktopRequest.applications.resource = resource;
                    }                   
                    if (model.getLogWatch)
                    {
                        LogWatch logWatch = new LogWatch();
                        logWatch.fullAccess = true;
                        desktopRequest.applications.logwatch = logWatch;
                    }
                    if (model.getTextEditor)
                    {
                        TextEditor textEditor = new TextEditor();
                        textEditor.fullAccess = true;
                        desktopRequest.applications.texteditor = textEditor;
                    }
                    response = await dwServiceDataStore.newSession(desktopRequest);
                    if(response.url == null)
                        return View("~/Views/Shared/Error.cshtml");
                    DesktopResponseViewModel viewModel = new DesktopResponseViewModel();
                    viewModel.idChannel = channel;
                    viewModel.url = response.url;
                    viewModel.canDesktop = model.getDesktop;
                    viewModel.canFileSystem = model.getFileSystem;
                    viewModel.canLogWatch = model.getLogWatch;
                    viewModel.canResoruce = model.getResoruce;
                    viewModel.canTextEditor = model.getTextEditor;
                    viewModel.fullAccess = model.fullAccess;

                    return RedirectToAction("NewSession", viewModel);

                }
                else
                    return View("NoFreeChannel");
            }
            catch (Exception ex)
            {

                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public ActionResult NewSession(DesktopResponseViewModel model)
        {
            return View(model);
        }

        public async Task<ActionResult> closeSession(int channelID)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                BaseResponse baseResponse = await dwServiceDataStore.closeSessionByIdChannel(channelID.ToString());
                if (baseResponse.status == 200)
                    return Json("OK");
                else
                    return Json("KO");
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public async Task<ActionResult> ListAllSessions()
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                SessionResponse model = await dwServiceDataStore.listSession();
                return View(model);
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public ActionResult GetSessionById()
        {
            return View();
        }

        [HttpPost]
        public ActionResult GetSessionById(string id)
        {
            if(string.IsNullOrEmpty(id))
                return View("~/Views/Shared/Error.cshtml");
            SessionRequest getSessionRequest = new SessionRequest();
            getSessionRequest.id = id;
            return RedirectToAction("GetSessionByIdResponse", getSessionRequest);
        }

        public async Task<ActionResult> GetSessionByIdResponse(SessionRequest getSessionRequest)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                SessionResponse response = await dwServiceDataStore.getSessionById(getSessionRequest.id);
                Session session = response.sessions.First();
                return View(session);
            }
            catch (Exception)
            {

                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public ActionResult GetSessionByIdChannel()
        {
            return View();
        }

        [HttpPost]
        public ActionResult GetSessionByIdChannel(int id)
        {
            SessionRequest getSessionRequest = new SessionRequest();
            getSessionRequest.idChannel = id;
            return RedirectToAction("GetSessionByIdChannelResponse", getSessionRequest);
        }

        public async Task<ActionResult> GetSessionByIdChannelResponse(SessionRequest getSessionRequest)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                SessionResponse response = await dwServiceDataStore.getSessionByIdChannel(getSessionRequest.idChannel.ToString());
                Session session = response.sessions.First();
                return View(session);
            }
            catch (Exception)
            {

                return View("~/Views/Shared/Error.cshtml");
            }
        }

        public ActionResult CloseSessionById()
        {
            return View();
        }

        [HttpPost]
        public async Task<ActionResult> CloseSessionById(string id)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                BaseResponse response = await dwServiceDataStore.closeSessionById(id);
                if (response.status == 200)
                {
                    return View("Index");
                }
                else
                {
                    return View("~/Views/Shared/Error.cshtml");

                }
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }

        }


        public ActionResult CloseSessionByIdChannel()
        {
            return View();
        }

        [HttpPost]
        public async Task<ActionResult> CloseSessionByIdChannel(int id)
        {
            try
            {
                DwServiceDataStore dwServiceDataStore = new DwServiceDataStore();
                BaseResponse response = await dwServiceDataStore.closeSessionByIdChannel(id.ToString());
                if (response.status == 200)
                {
                    return View("Index");
                }
                else
                {
                    return View("~/Views/Shared/Error.cshtml");

                }
            }
            catch (Exception)
            {
                return View("~/Views/Shared/Error.cshtml");
            }

        }



    }
}