﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using DemoApiDwservice.Models.Request;
using DemoApiDwservice.Models.Response;
using DemoApiDwservice.Models;
using System.Security.Cryptography;
using System.IO;
using System.Security.Authentication;

namespace DemoApiDwservice.Services
{
    public class DwServiceDataStore
    {
        HttpClient client;
        DateTime Jan1St1970 = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);

        public DwServiceDataStore()
        {
            client = new HttpClient();
            client.BaseAddress = new Uri($"https://www.apiremoteaccess.com/en/api/json/");
            string key = AppSettings.key;
            string secret = AppSettings.secret;
            string authtype = "hmac1";
            if (authtype == "")
            {
                client.DefaultRequestHeaders.Authorization =
        new AuthenticationHeaderValue(
        "Basic", Convert.ToBase64String(
           ASCIIEncoding.ASCII.GetBytes(
               $"{key}:{secret}")));
            }
            else if (authtype == "hmac1") {
                long ctime = (long)((DateTime.UtcNow - Jan1St1970).TotalMilliseconds);

                HMACSHA1 myhmacsha1 = new HMACSHA1(Encoding.ASCII.GetBytes(secret));
                byte[] byteArray = Encoding.ASCII.GetBytes(ctime + "");
                MemoryStream stream = new MemoryStream(byteArray);
                byte[] hashValue = myhmacsha1.ComputeHash(stream);
                string shmac = Convert.ToBase64String(hashValue);
                client.DefaultRequestHeaders.Add("X-DW-Authorization", "HM1 " + key + ":" + shmac + ":" + ctime);

            }
            else
            {
                
            }
        }

        public async Task<SessionResponse> listSession()
        {
            SessionResponse response = new SessionResponse();
            Session[] sessions;
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"sessions"),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();

                if (contents.Length > 2)
                {
                    sessions = await Task.Run(() => JsonConvert.DeserializeObject<Session[]>(contents));
                    response.sessions = sessions;
                }


            }
            catch (Exception ex)
            {

                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<SessionResponse> getSessionById(string id)
        {
            SessionResponse response = new SessionResponse();
            Session sessions;
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"sessions?id=" + id),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();

                if (contents.Length > 2)
                {
                    sessions = await Task.Run(() => JsonConvert.DeserializeObject<Session>(contents));
                    response.sessions = new Session[1];
                    response.sessions[0] = sessions;
                }


            }
            catch (Exception ex)
            {

                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<SessionResponse> getSessionByIdChannel(string id)
        {
            SessionResponse response = new SessionResponse();
            Session sessions;
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"sessions?idChannel=" + id),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();

                if (contents.Length > 2)
                {
                    sessions = await Task.Run(() => JsonConvert.DeserializeObject<Session>(contents));
                    response.sessions = new Session[1];
                    response.sessions[0] = sessions;
                }


            }
            catch (Exception ex)
            {

                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }


        public async Task<DesktopResponse> newSession(DesktopRequest request)
        {
            DesktopResponse response = new DesktopResponse();
            try
            {

                JsonSerializerSettings settings = new JsonSerializerSettings() { NullValueHandling = NullValueHandling.Ignore };

                string jsonRequest = JsonConvert.SerializeObject(request, settings);

                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var jsonResponse = await client.PostAsync($"sessions", content);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<DesktopResponse>(contents));
            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }

            return response;
        }

        public async Task<Agent> newAgent(AgentRequest request)
        {
            Agent response = null;
            try
            {
                string jsonRequest = JsonConvert.SerializeObject(request);
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var jsonResponse = await client.PostAsync($"agents", content);
                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<Agent>(contents));
            }
            catch (Exception ex)
            {
                //
            }
            return response;
        }

        public async Task<BaseResponse> deleteAgent(string id)
        {
            BaseResponse response = new BaseResponse();
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Delete,
                    RequestUri = new Uri(client.BaseAddress + $"agents?id=" + id),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<BaseResponse>(contents));

            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<BaseResponse> closeSessionById(string sessionId)
        {
            BaseResponse response = new BaseResponse();
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Delete,
                    RequestUri = new Uri(client.BaseAddress + $"sessions?id=" + sessionId),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<BaseResponse>(contents));

            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<BaseResponse> closeSessionByIdChannel(string idChannel)
        {
            BaseResponse response = new BaseResponse();
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Delete,
                    RequestUri = new Uri(client.BaseAddress + $"sessions?idChannel=" + idChannel),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<BaseResponse>(contents));

            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<BaseResponse> modifyAgent(string id, AgentRequest request)
        {
            BaseResponse response = new BaseResponse();
            try
            {

                string jsonRequest = JsonConvert.SerializeObject(request);
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Put,
                    RequestUri = new Uri(client.BaseAddress + $"agents?id=" + id),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<BaseResponse>(contents));

            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<Account> getAccountInfo()
        {
            Account response = new Account();

            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"account?request=info"),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<Account>(contents));

            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<List<TransactionsResponse>> getCreditTransactions(int days)
        {
            List<TransactionsResponse> response = new List<TransactionsResponse>();

            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"account?request=creditTransactions&days=" + days.ToString()),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<List<TransactionsResponse>>(contents));

            }
            catch (Exception ex)
            {
                throw;
            }
            return response;
        }

        public async Task<ServiceResponse> addService(ServiceRequest request)
        {
            ServiceResponse response = new ServiceResponse();
            try
            {

                string jsonRequest = JsonConvert.SerializeObject(request);
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Put,
                    RequestUri = new Uri(client.BaseAddress + $"account?request=addServices"),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<ServiceResponse>(contents));
                response.StatusCode = jsonResponse.StatusCode;

            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }
        public async Task<RemoveServiceResponse> removeService(RemoveServiceRequest request)
        {
            RemoveServiceResponse response = new RemoveServiceResponse();
            try
            {
                JsonSerializerSettings settings = new JsonSerializerSettings() { NullValueHandling = NullValueHandling.Ignore };

                string jsonRequest = JsonConvert.SerializeObject(request, settings);

                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Put,
                    RequestUri = new Uri(client.BaseAddress + $"account?request=removeServices"),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);

                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<RemoveServiceResponse>(contents));
                response.StatusCode = jsonResponse.StatusCode;

            }
            catch (Exception ex)
            {
                response.status = 500;
                response.description = ex.Message;
            }
            return response;
        }

        public async Task<List<Agent>> getAgentsList()
        {
            List<Agent> response = null;
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"agents"),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);



                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<List<Agent>>(contents));

            }
            catch (Exception ex)
            {
                throw ex;
            }
            return response;
        }

        public async Task<Agent> getAgentbyId(string id)
        {
            Agent response = new Agent();
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"agents?id=" + id),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);



                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<Agent>(contents));

            }
            catch (Exception ex)
            {
                throw ex;
            }
            return response;
        }

        public async Task<Agent> getAgentbyName(string name)
        {
            Agent response = new Agent();
            try
            {

                string jsonRequest = JsonConvert.SerializeObject("");
                StringContent content = new StringContent(jsonRequest, Encoding.UTF8, "application/json");

                var Request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(client.BaseAddress + $"agents?name=" + name),
                    Content = content
                };


                var jsonResponse = await client.SendAsync(Request);



                var contents = await jsonResponse.Content.ReadAsStringAsync();
                response = await Task.Run(() => JsonConvert.DeserializeObject<Agent>(contents));

            }
            catch (Exception ex)
            {
                throw ex;
            }
            return response;
        }
    }
}
